# Copyright (C) Tal Galili
#
# This file is part of dendextend.
#
# dendextend is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# dendextend is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#





#' Functions for extending dendrogram objects
#'
#' @name dendextend-package
#' @aliases dendextend
#' @description 
#' Offers a set of functions for extending 'dendrogram' objects in R, 
#' letting you visualize and compare trees of 'hierarchical clusterings'.
#' You can (1) Adjust a tree's graphical parameters - 
#' the color, size, type, etc of its branches, nodes and labels.
#' (2) Visually and statistically compare different 'dendrograms' to one another.
#' 
#' @seealso \code{\link{dendrogram}, \link{hclust}} in \code{\link{stats}} package.
#' @docType package
#' @keywords package dendrogram clustering visualization
#' @import stats
#' @import utils
#' @import graphics
#' @import grDevices
#' @import magrittr
NULL

# help(package = "dendextend")
