# Copyright (C) Tal Galili
#
# This file is part of dendextend.
#
# dendextend is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# dendextend is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#



#' @importFrom magrittr %>%
#' @export
magrittr::`%>%`


# For making this decision, I followed the text from:
# https://cran.r-project.org/doc/manuals/r-release/R-exts.html#Package-structure
# https://cran.r-project.org/package=dplyr
# https://cran.r-project.org/package=magrittr
# https://CRAN.R-project.org/package=roxygen2/vignettes/rd.html
# https://CRAN.R-project.org/package=roxygen2/vignettes/namespace.html
# https://stackoverflow.com/questions/8637993/better-explanation-of-when-to-use-imports-depends
